!!   ###############################
SUBROUTINE INIT_MGN2MECH(HMECHANISM, OCONVERSION, HVNAME3D, HMECH_SPC, &
                         KSPMH_MAP, KMECH_MAP, PCONV_FAC, PMECH_MWT,   &
                         KVARS3D, K_SCON_SPC)
!!
!!***  *BVOCEM*
!! 
!!    PURPOSE
!!    -------
!!    CALCULATE THE BIOGENIC EMISSION FLUXES UPON THE MEGAN CODE
!!    HTTP://LAR.WSU.EDU/MEGAN/
!!
!!    METHOD
!!    ------
!!
!!
!!    AUTHOR
!!    ------
!   ORIGINALLY CREATED BY JACK CHEN 11/04 FOR MEGAN V.0
!   FOR MEGAN V2.0 CREATED BY TAN 12/01/06
!   FOR MEGAN V2.1 CREATED BY XUEMEI WANG 11/04/07
!   FOR MEGAN V2.1 TO USE 150 SPECIES CREATED BY XUEMEI WANG 09/30/09
!
!  HISTORY:
!  08/14/07 TAN    - MOVE TO MEGANV2.02 WITH NO UPDATE
!  08/29/07 MODIFIED BY A. GUENTHER TO CORRECT ERROR IN ASSIGNING
!           EMISSION FACTOR. THIS VERSION IS CALLED MEGANV2.03
!  10/29/07 MODIFIED BY A. GUENTHER TO CORRECT OMISSION OF DIURNAL VARIATION
!           FACTOR. THIS VERSION IS CALLED MEGANV2.04
!  11/04/07 MODIFIED BY XUEMEI WANG TO GIVE TWO OPTIONS FOR MAP OR LOOKUP TABLE FOR
!           THE EMISSION FACTORS. ALSO GIVES OPTIONS FOR DIFFERENT CHEMICAL MECHANISMS
!           IN THE CODE: USER MODIFIES THE EXTERNAL SCRIPT TO ASSIGN MECHANISM.
!           THIS VERSION IS CALLED MEGANV2.1.0
!  06/04/08 MODIFIED BY J. LEE-TAYLOR TO ACCEPT VEGETATION-DEPENDENT SPECIATION FACTORS
!           IN TABLE FORMAT (RESHAPE TABLES) RATHER THAN FROM DATA STATEMENTS.
!  09/30/08  MODIFIED BY XUEMEI WANG TO GIVE OPTIONS FOR INPUT FILE AND TEST DIFFERENT MECHANISMS
!  09/27/11  TAN&XUEMEI MEGANV2.10 INCLUDES SOIL NOX ADJUSTMENT AND A LOT OF UPDATES
!  20/12/14  P. TULET - ON-LINE COUPLING IN THE ISBA/SURFEX SCHEME. ALL INIT VARIABLES HAS BEEN
!            MOVED IN INIT_MEGANN.F90.
!!    
!!    MODIFICATIONS
!!    -------------
!!    ORIGINAL: 25/10/14
!!
!!
!!    EXTERNAL
!!    --------
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!
!------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!        -----------------
!
USE MODD_MGN2MECH
!
IMPLICIT NONE

CHARACTER(LEN=16), INTENT(IN) :: HMECHANISM !I MECHANISM NAME
LOGICAL, INTENT(IN) :: OCONVERSION !I 
!
CHARACTER(LEN=16),DIMENSION(:), POINTER :: HMECH_SPC!I MECHANISM NAME
CHARACTER(LEN=16),DIMENSION(:), POINTER :: HVNAME3D !I MECHANISM NAME
INTEGER,DIMENSION(:), POINTER :: KSPMH_MAP
INTEGER,DIMENSION(:), POINTER :: KMECH_MAP
REAL,DIMENSION(:), POINTER :: PCONV_FAC
REAL,DIMENSION(:), POINTER :: PMECH_MWT
INTEGER, INTENT(INOUT) :: KVARS3D
INTEGER, INTENT(INOUT) :: K_SCON_SPC

!...  INCLUDES:
!
!*   0.1 DECLARATION OF LOCAL VARIABLES
!
! SET ATTRIBUTE AND VARIABLES FOR OUTPUT
SELECT CASE ( TRIM(HMECHANISM) )
  CASE ('CB05')
    K_SCON_SPC = N_CB05
    KVARS3D = N_CB05_SPC
  CASE ('CB6')
    K_SCON_SPC = N_CB6
    KVARS3D = N_CB6_SPC
  CASE ('SAPRCII')
    K_SCON_SPC = N_SAPRCII
    KVARS3D = N_SAPRCII_SPC
  CASE ('RADM2')
    K_SCON_SPC = N_RADM2
    KVARS3D = N_RADM2_SPC
  CASE ('RACM')
    K_SCON_SPC = N_RACM
    KVARS3D = N_RACM_SPC
  CASE ('CBMZ')
    K_SCON_SPC = N_CBMZ
    KVARS3D = N_CBMZ_SPC    
  CASE ('SAPRC99')
    K_SCON_SPC = N_SAPRC99
    KVARS3D = N_SAPRC99_SPC
  CASE ('SAPRC99Q')
    K_SCON_SPC = N_SAPRC99_Q
    KVARS3D = N_SAPRC99_Q_SPC    
  CASE ('SAPRC99X')
    K_SCON_SPC = N_SAPRC99_X
    KVARS3D = N_SAPRC99_X_SPC
  CASE ('SOAX')
    K_SCON_SPC = N_SOAX
    KVARS3D = N_SOAX_SPC
  CASE DEFAULT
    CALL ABOR1_SFX("ERROR: MECHANISM CONVERSION, INVALID MECHANISM: "//TRIM(HMECHANISM))
ENDSELECT

!  PRINT*,'SHAPE(SPMH_MAP) =',SHAPE(SPMH_MAP)
IF (ASSOCIATED(KSPMH_MAP)) DEALLOCATE(KSPMH_MAP)
ALLOCATE(KSPMH_MAP(K_SCON_SPC))

IF (ASSOCIATED(KMECH_MAP)) DEALLOCATE(KMECH_MAP)
ALLOCATE(KMECH_MAP(K_SCON_SPC))

IF (ASSOCIATED(PCONV_FAC)) DEALLOCATE(PCONV_FAC)
ALLOCATE(PCONV_FAC(K_SCON_SPC))

IF (ASSOCIATED(HMECH_SPC)) DEALLOCATE(HMECH_SPC)
ALLOCATE(HMECH_SPC(KVARS3D))

IF (ASSOCIATED(PMECH_MWT)) DEALLOCATE(PMECH_MWT)
ALLOCATE(PMECH_MWT(KVARS3D))

IF (ASSOCIATED(HVNAME3D)) DEALLOCATE(HVNAME3D)
ALLOCATE(HVNAME3D(KVARS3D))

IF ( OCONVERSION ) THEN

  SELECT CASE ( TRIM(HMECHANISM) )
    CASE ('CB05')
      KSPMH_MAP(:) = NSPMH_MAP_CB05(:)
      KMECH_MAP(:) = NMECH_MAP_CB05(:)
      PCONV_FAC(:) = XCONV_FAC_CB05(:)
      HMECH_SPC(:) = CMECH_SPC_CB05(:)
      PMECH_MWT(:) = XMECH_MWT_CB05(:)
    CASE ('CB6')
      KSPMH_MAP(:) = NSPMH_MAP_CB6(:)
      KMECH_MAP(:) = NMECH_MAP_CB6(:)
      PCONV_FAC(:) = XCONV_FAC_CB6(:)
      HMECH_SPC(:) = CMECH_SPC_CB6(:)
      PMECH_MWT(:) = XMECH_MWT_CB6(:)      
    CASE ('SAPRCII')
      KSPMH_MAP(:) = NSPMH_MAP_SAPRCII(:)
      KMECH_MAP(:) = NMECH_MAP_SAPRCII(:)
      PCONV_FAC(:) = XCONV_FAC_SAPRCII(:)
      HMECH_SPC(:) = CMECH_SPC_SAPRCII(:)
      PMECH_MWT(:) = XMECH_MWT_SAPRCII(:)
    CASE ('RADM2')
      KSPMH_MAP(:) = NSPMH_MAP_RADM2(:)
      KMECH_MAP(:) = NMECH_MAP_RADM2(:)
      PCONV_FAC(:) = XCONV_FAC_RADM2(:)
      HMECH_SPC(:) = CMECH_SPC_RADM2(:)
      PMECH_MWT(:) = XMECH_MWT_RADM2(:)
    CASE ('RACM')
      KSPMH_MAP(:) = NSPMH_MAP_RACM(:)
      KMECH_MAP(:) = NMECH_MAP_RACM(:)
      PCONV_FAC(:) = XCONV_FAC_RACM(:)
      HMECH_SPC(:) = CMECH_SPC_RACM(:)
      PMECH_MWT(:) = XMECH_MWT_RACM(:)
    CASE ('CBMZ')
      KSPMH_MAP(:) = NSPMH_MAP_CBMZ(:)
      KMECH_MAP(:) = NMECH_MAP_CBMZ(:)
      PCONV_FAC(:) = XCONV_FAC_CBMZ(:)
      HMECH_SPC(:) = CMECH_SPC_CBMZ(:)
      PMECH_MWT(:) = XMECH_MWT_CBMZ(:)
    CASE ('SAPRC99')
      KSPMH_MAP(:) = NSPMH_MAP_SAPRC99(:)
      KMECH_MAP(:) = NMECH_MAP_SAPRC99(:)
      PCONV_FAC(:) = XCONV_FAC_SAPRC99(:)
      HMECH_SPC(:) = CMECH_SPC_SAPRC99(:)
      PMECH_MWT(:) = XMECH_MWT_SAPRC99(:)
    CASE ('SAPRC99Q')
      KSPMH_MAP(:) = NSPMH_MAP_SAPRC99_Q(:)
      KMECH_MAP(:) = NMECH_MAP_SAPRC99_Q(:)
      PCONV_FAC(:) = XCONV_FAC_SAPRC99_Q(:)
      HMECH_SPC(:) = CMECH_SPC_SAPRC99_Q(:)
      PMECH_MWT(:) = XMECH_MWT_SAPRC99_Q(:)
    CASE ('SAPRC99X')
      KSPMH_MAP(:) = NSPMH_MAP_SAPRC99_X(:)
      KMECH_MAP(:) = NMECH_MAP_SAPRC99_X(:)
      PCONV_FAC(:) = XCONV_FAC_SAPRC99_X(:)
      HMECH_SPC(:) = CMECH_SPC_SAPRC99_X(:)
      PMECH_MWT(:) = XMECH_MWT_SAPRC99_X(:)
    CASE ('SOAX')
      KSPMH_MAP(:) = NSPMH_MAP_SOAX(:)
      KMECH_MAP(:) = NMECH_MAP_SOAX(:)
      PCONV_FAC(:) = XCONV_FAC_SOAX(:)
      HMECH_SPC(:) = CMECH_SPC_SOAX(:)
      PMECH_MWT(:) = XMECH_MWT_SOAX(:)
  ENDSELECT

  HVNAME3D(:)  = HMECH_SPC(:)

ELSE

  KVARS3D = N_SPCA_SPC
  HVNAME3D(:) = CSPCA_SPC(:)
  
ENDIF

!---------------------------------------------------------------------------
!
END SUBROUTINE INIT_MGN2MECH
